#pragma once

#include "Slate.h"
#include "SSkillButton.h"
#include "SkillsPanelStyle.h"
#include "GUIStyle.h"

/** SSkillsPanel is a container that holds SSkillButtons. Use one of the AddButton function to do it.
*	Default style should be placed under Content/UI and named SkillsPanel_Default.*/
class GUILIBRARY_API SSkillsPanel : public SCompoundWidget
{
public:
	SLATE_BEGIN_ARGS(SSkillsPanel)
		: _Orientation(EOrientation::Orient_Horizontal)
		, _ButtonHeight(50)
		, _ButtonWidth(50)
		, _SkillsPanelStyle(&FGUIStyle::Get().GetWidgetStyle<FSkillsPanelWidgetStyle>("SkillsPanel_Default"))
		, _ButtonsPadding(FMargin(4))
	{}
		/* Style of this widget. */
		SLATE_STYLE_ARGUMENT(FSkillsPanelWidgetStyle, SkillsPanelStyle)
		/* List needed to execute commands mapped to buttons  */
		SLATE_ARGUMENT(TSharedPtr<FUICommandList>, CommandList)
		/* Orientation of this panel. Default: Horizontal */
		SLATE_ARGUMENT(EOrientation, Orientation)
		/* Minimum button height. Default: 50 */
		SLATE_ARGUMENT(float, ButtonHeight)
		/* Minimum button width. Default: 50 */
		SLATE_ARGUMENT(float, ButtonWidth)
		/* Buttons Padding. Default: 4 */
		SLATE_ARGUMENT(FMargin, ButtonsPadding)
	SLATE_END_ARGS()

		void Construct(const FArguments& InArgs);

	/*
	*	Add button to panel.
	*	@param InCommand Command that should be executed when button is pressed.
	*	NOTE: Button will display SlateIcon bound to InCommand on top of everything else.
	*	Also, if there is InputGesture bound to command it will be used as Hotkey text.
	*/
	void AddSkillButton(TSharedRef<FUICommandInfo> InCommand);

	/*
	*	Add button to panel.
	*	@param InCommand Command that should be executed when button is pressed.
	*	@param InButtonStyle Style that should be applied to this button.
	*	NOTE: Button will display SlateIcon bound to InCommand on top of everything else.
	*	Also, if there is InputGesture bound to command it will be used as Hotkey text.
	*/
	void AddSkillButton(TSharedRef<FUICommandInfo> InCommand, const FSkillButtonWidgetStyle* InButtonStyle);

	/*
	*	Add button to panel.
	*	@param OnButtonClick Simple delegate that will be executed when button is pressed.
	*/
	void AddSkillButton(FSimpleDelegate OnButtonClick);

	/*
	*	Add button to panel.
	*	@param OnButtonClick Simple delegate that will be executed when button is pressed.
	*	@param KeyText Text that should be displayed as "Hotkey" under the button.
	*/
	void AddSkillButton(FSimpleDelegate OnButtonClick, FString KeyText);

	/*
	*	Add button to panel.
	*	@param InAction Action that will be executed when button is pressed. Button will also use 'CanExecute' to determine if it should be disabled or not.
	*	@param KeyText Text that should be displayed as "Hotkey" under the button.
	*/
	void AddSkillButton(FUIAction InAction, FString KeyText = FString(""));

	/*
	*	Add button to panel.
	*	@param OnButtonClick Simple delegate that will be executed when button is pressed.
	*	@param InButtonStyle Style that should be applied to this button.
	*/
	void AddSkillButton(FSimpleDelegate OnButtonClick, const FSkillButtonWidgetStyle* InButtonStyle);

	/*
	*	Show/Hide hotkey text of the buttons.
	*	@param bShow true if you want to show Hotkey text, false otherwise.
	*/
	void ShowHotkeysText(bool bShow);

protected:

	/*
	*	Add button widget to display list.
	*	@param InButton Button to add.
	*/
	void AddButton(TSharedRef<SSkillButton> InButton);

	/*
	*	Get button style.
	*	@return Return value from Styles array or default ButtonStyle.
	*/
	const FSkillButtonWidgetStyle* GetButtonStyle();

	/* Container for buttons. */
	TSharedPtr<SUniformGridPanel> SkillsContainer;

	/* Buttons counter. */
	int32 SlotCounter;

	/* Array of active buttons. */
	TArray<TSharedPtr<SSkillButton>> ActiveButtons;

	/* List needed to execute commands mapped to buttons. */
	TSharedPtr<FUICommandList> CommandList;

	/* Orientation. Horizontal by default. */
	EOrientation Orientation;

	/* Should buttons on this panel draw Hotkey text? */
	bool bDrawHotkeyText;

	/* Should buttons on this panel draw Hotkey text inside themselves? */
	bool bHotkeyTextInsideButton;

	/* Default button style used for buttons added to this panel. Note that it wont be used if there are styles defined in Style->SkillButtonStyles.  */
	const FSkillButtonWidgetStyle* ButtonStyle;

	/* Style of this panel. */
	const FSkillsPanelWidgetStyle* Style;
};